package fr.soleil.docking.infonode.view;

import java.awt.Color;
import java.awt.Component;
import java.util.List;

import javax.swing.Icon;

import net.infonode.docking.DockingWindow;
import net.infonode.docking.DockingWindowListener;
import net.infonode.docking.OperationAbortedException;
import net.infonode.docking.View;
import fr.soleil.docking.view.IView;
import fr.soleil.docking.view.IViewListener;

/**
 * A dynamically created view containing an id.
 * 
 * @author Hardion
 * @author GIRARDOT
 */
public class InfoNodeView extends View implements IView {

    private static final long serialVersionUID = -8766019787338480450L;

    protected Object id;

    /**
     * Constructor.
     * 
     * @param title the view title
     * @param icon the view icon
     * @param component the view component
     * @param id the view id
     */
    public InfoNodeView(String title, Icon icon, Component component, Object id) {
        super(title, icon, component);
        this.id = id;
    }

    @Override
    public Object getId() {
        return id;
    }

    @Override
    public boolean isVisible() {
        boolean visible;
        if (super.isShowing()) {
            visible = true;
        } else if (super.getWindowParent() != null && super.getWindowParent().isRestorable()) {
            visible = true;
        } else {
            visible = false;
        }
        return visible;
    }

    @Override
    public void setVisible(boolean visible) {
        if (visible) {
            super.restore();
        } else {
            super.close();
        }
        super.setVisible(visible);
        super.setEnabled(visible);
    }

    @Override
    protected void update() {
        super.update();
        super.setVisible(true);
        super.setEnabled(true);

    }

    @Override
    public Color getViewBackground() {
        return getBackground();
    }

    @Override
    public void setViewBackground(Color bg) {
        setComponentsBackground(getCustomTabComponents(), bg);
        setComponentsBackground(getCustomTitleBarComponents(), bg);
        setBackground(this, bg);
    }

    @Override
    public Color getViewForeground() {
        return getForeground();
    }

    @Override
    public void setViewForeground(Color fg) {
        setComponentsForeground(getCustomTabComponents(), fg);
        setComponentsForeground(getCustomTitleBarComponents(), fg);
        setForeground(this, fg);
    }

    /**
     * Sets the background {@link Color} of a {@link DockingWindow}
     * 
     * @param win The {@link DockingWindow}
     * @param bg The background {@link Color} to set
     */
    protected void setBackground(DockingWindow win, Color bg) {
        win.setBackground(bg);
        win.setBorder(null);
        win.getWindowProperties().getTabProperties().getTitledTabProperties().getNormalProperties()
        .getComponentProperties().setBackgroundColor(bg);
        win.getWindowProperties().getTabProperties().getTitledTabProperties().getHighlightedProperties()
        .getComponentProperties().setBackgroundColor(bg);
    }

    /**
     * Sets the foreground {@link Color} of a {@link DockingWindow}
     * 
     * @param win The {@link DockingWindow}
     * @param fg The foreground {@link Color} to set
     */
    protected void setForeground(DockingWindow win, Color fg) {
        win.setForeground(fg);
        win.getWindowProperties().getTabProperties().getTitledTabProperties().getNormalProperties()
        .getComponentProperties().setForegroundColor(fg);
        win.getWindowProperties().getTabProperties().getTitledTabProperties().getHighlightedProperties()
        .getComponentProperties().setForegroundColor(fg);
    }

    /**
     * Sets the background of all {@link Component}s in a given {@link List}
     * 
     * @param components The {@link List}
     * @param bg The background to set
     */
    protected void setComponentsBackground(List<?> components, Color bg) {
        if (components != null) {
            for (Object comp : components) {
                if (comp instanceof Component) {
                    ((Component) comp).setBackground(bg);
                }
            }
        }
    }

    /**
     * Sets the foreground of all {@link Component}s in a given {@link List}
     * 
     * @param components The {@link List}
     * @param bg The foreground to set
     */
    protected void setComponentsForeground(List<?> components, Color bg) {
        if (components != null) {
            for (Object comp : components) {
                if (comp instanceof Component) {
                    ((Component) comp).setForeground(bg);
                }
            }
        }
    }

    @Override
    public void setClosable(boolean closable) {
        if (getWindowProperties() != null) {
            getWindowProperties().getTabProperties().getNormalButtonProperties().getCloseButtonProperties()
            .setVisible(closable);
            getWindowProperties().getTabProperties().getHighlightedButtonProperties().getCloseButtonProperties()
            .setVisible(closable);
            getWindowProperties().setCloseEnabled(closable);
        }
    }

    @Override
    public void addViewListener(final IViewListener listener) {
        addListener(new DockingWindowListener() {

            @Override
            public void windowUndocking(DockingWindow window) throws OperationAbortedException {
                // Not used yet.
            }

            @Override
            public void windowUndocked(DockingWindow window) {
                // Not used yet.
            }

            @Override
            public void windowShown(DockingWindow window) {
                // Not used yet.
            }

            @Override
            public void windowRestoring(DockingWindow window) throws OperationAbortedException {
                // Not used yet.
            }

            @Override
            public void windowRestored(DockingWindow window) {
                // Not used yet.
            }

            @Override
            public void windowRemoved(DockingWindow removedFromWindow, DockingWindow removedWindow) {
                // Not used yet.
            }

            @Override
            public void windowMinimizing(DockingWindow window) throws OperationAbortedException {
                // Not used yet.
            }

            @Override
            public void windowMinimized(DockingWindow window) {
                // Not used yet.
            }

            @Override
            public void windowMaximizing(DockingWindow window) throws OperationAbortedException {
                // Not used yet.
            }

            @Override
            public void windowMaximized(DockingWindow window) {
                // Not used yet.
            }

            @Override
            public void windowHidden(DockingWindow window) {
                // Not used yet.
            }

            @Override
            public void windowDocking(DockingWindow window) throws OperationAbortedException {
                // Not used yet.
            }

            @Override
            public void windowDocked(DockingWindow window) {
                // Not used yet.
            }

            @Override
            public void windowClosing(DockingWindow window) throws OperationAbortedException {
                // Not used yet.
            }

            @Override
            public void windowClosed(DockingWindow window) {
                listener.viewClosed();
            }

            @Override
            public void windowAdded(DockingWindow addedToWindow, DockingWindow addedWindow) {
                // Not used yet.
            }

            @Override
            public void viewFocusChanged(View previouslyFocusedView, View focusedView) {
                if (InfoNodeView.this.equals(focusedView)) {
                    listener.focusGained();
                } else {
                    listener.focusLost();
                }
            }
        });

    }

}
